/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "lcrzoex.h"
#include "lcrzoex__00001.h"
#include "lcrzoex__00002.h"
#include "lcrzoex__00003.h"
#include "lcrzoex__00004.h"
#include "lcrzoex__00005.h"
#include "lcrzoex__00006.h"
#include "lcrzoex__00007.h"

/*-------------------------------------------------------------*/
/* function declaration */
int main(int argc, char* argv[]);
int lcrzoexmain(int argc, char* argv[]);
int lcrzoex_check_ending_number(lcrzo_conststring progname,
                                lcrzo_bool *phasanendingnumber,
                                lcrzo_int32 *pexnumber);
int lcrzoex_helpchoice(lcrzo_conststring progname);
int lcrzoex_helpchoice_tool_cat(lcrzo_int16 language);
int lcrzoex_helpchoice_tool_subcat(lcrzo_int16 language);
int lcrzoex_helpchoice_tool_choose(lcrzo_int16 language,
				   lcrzo_string levels,
				   lcrzo_int16 level,
				   lcrzo_uint32 minnum,
				   lcrzo_uint32 maxnum,
				   lcrzo_uint32 *ptoolnumber);
int lcrzoex_helpchoice_tool_list(lcrzo_string levels, 
				 lcrzo_uint32 *ptoolnumber);
int lcrzoex_helpchoice_subcat(lcrzo_int16 language);
int lcrzoex_helpchoice_subcat_choose(lcrzo_int16 language,
				     lcrzo_string levels,
				     lcrzo_int16 level,
				     char lastchar,
				     lcrzo_uint8 *pchoice);

/*-------------------------------------------------------------*/
/* main function */
int main(int argc, char* argv[])
{ int ret;

  /* initialize lcrzo */
  lcrzo_init(); 

  /* to check lcrzo compilation parameters*/
  lcrzo_epr(lcrzo_compilparam_check());

  /* if you want to change language */
  /*lcrzo_epr(lcrzo_global_set_language(LCRZO_GLOBAL_LANG_FR));*/
  /*lcrzo_epr(lcrzo_global_set_language(LCRZO_GLOBAL_LANG_EN));*/

  /* the real main function */
  ret=lcrzoexmain(argc, argv);
  
  /* close lcrzo */
  lcrzo_close();
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoexmain(int argc, char* argv[])
{ lcrzo_int32 exnumber;
  lcrzo_int16 language;
  lcrzo_bool hasanendingnumber;
  int ac, ret;
  char **av;
  
  /* add error displaying functions */
  lcrzo_epr(lcrzo_global_set_err_displayfunction(&lcrzoex_err_print));

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* check if the program contains an ending number */
  lcrzo_epr(lcrzoex_check_ending_number(argv[0], &hasanendingnumber,
					&exnumber));

  /* if it does not end with a number, read it from the command line */
  if (!hasanendingnumber)
  { /* we help user to choose the tool he wants */
    if ( argc==1 )
    { return(lcrzoex_helpchoice(argv[0]));
    }
    lcrzo_epr(lcrzo_int32_init_text(argv[1], &exnumber));
    ac=argc-1;
    av=argv+1;
  }
  else
  { ac=argc;
    av=argv;
  }

  /* call the wanted tool */
  ret=LCRZO_ERR_OK;
  if ( exnumber>=1 && exnumber<=50 )
  { ret=lcrzoex__00001_main(ac, av, exnumber);
  }
  else if ( exnumber>=51 && exnumber<=100 )
  { ret=lcrzoex__00002_main(ac, av, exnumber);
  }
  else if ( exnumber>=101 && exnumber<=150 )
  { ret=lcrzoex__00003_main(ac, av, exnumber);
  }
  else if ( exnumber>=151 && exnumber<=200 )
  { ret=lcrzoex__00004_main(ac, av, exnumber);
  }
  else if ( exnumber>=201 && exnumber<=250 )
  { ret=lcrzoex__00005_main(ac, av, exnumber);
  }
  else if ( exnumber>=251 && exnumber<=300 )
  { ret=lcrzoex__00006_main(ac, av, exnumber);
  }
  else if ( exnumber>=301 && exnumber<=350 )
  { ret=lcrzoex__00007_main(ac, av, exnumber);
  }
  else 
  { if ( language==LCRZO_GLOBAL_LANG_FR )
    { lcrzo_int32_print("Choix '", exnumber, "%d", "' inconnu.\n");
      lcrzo_int32_print("L'outil numero ", exnumber, "%d",
			" est peut etre disponible ");
      printf("dans une version recente de lcrzoex.\n"); 
    }
    else
    { lcrzo_int32_print("Choice '", exnumber, "%d", "' unknown.\n");
      lcrzo_int32_print("Tool number ", exnumber, "%d",
			" is prehaps only available ");
      printf("in a newer version of lcrzoex.\n"); 
    }
    return(LCRZO_ERR_OK);
  }

  /* reinit lcrzo because tools might have closed it */
  lcrzo_init();
  lcrzo_epr(lcrzo_global_set_err_displayfunction(&lcrzoex_err_print));
  lcrzo_ep(ret);

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzoex_check_ending_number(lcrzo_conststring progname,
                                lcrzo_bool *phasanendingnumber,
                                lcrzo_int32 *pexnumber)
{ lcrzo_conststring pc;
  char c;
  int ret, i;

  if (phasanendingnumber!=NULL) *phasanendingnumber=LCRZO_FALSE;

  /* program name is too short */
  if (!strlen(progname)) return(LCRZO_ERR_OK);

  /* does not end with a digit, so exit now */
  c=progname[strlen(progname)-1];
  if ( c<'0' || c>'9' ) return(LCRZO_ERR_OK);

  /* find the position of the first non digit character */
  pc=progname;
  for ( i=strlen(progname)-1 ; i>=0 ; i-- )
  { c=progname[i];
    if ( c<'0' || c>'9' )
    { pc=progname+i+1;
      break;
    }
  }
  
  /* now, remove leading zeros */
  while ( strlen(pc) )
  { if ( *pc=='0' ) pc++;
    else break;
  }

  /* try to convert to a number */
  ret=lcrzo_int32_init_text(pc, pexnumber);
  if ( ret==LCRZO_ERR_OK)
  { if (phasanendingnumber!=NULL) *phasanendingnumber=LCRZO_TRUE;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_subcat(lcrzo_int16 language)
{
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("\n********** Plusieurs sous-categories sont disponibles :\n");
    printf(" 0 - revenir a la categorie precedente\n");
  }
  else 
  { printf("\n********** Several sub-categories are available :\n");
    printf(" 0 - go back to the previous category\n");  
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_tool_cat(lcrzo_int16 language)
{
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("\n++++++++++ Plusieurs outils sont disponibles dans cette categorie :\n");
    printf("     0 - revenir au menu principal\n");
  }
  else 
  { printf("\n++++++++++ Several tools are available in this category :\n");
    printf("     0 - go back to the main menu\n");  
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_tool_subcat(lcrzo_int16 language)
{
  if ( language==LCRZO_GLOBAL_LANG_FR )
  { printf("\n++++++++++ Plusieurs outils sont disponibles dans cette sous-categorie :\n");
    printf("     0 - revenir au menu principal\n");
  }
  else 
  { printf("\n++++++++++ Several tools are available in this sub-category :\n");
    printf("     0 - go back to the main menu\n");  
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_tool_choose(lcrzo_int16 language,
				   lcrzo_string levels, 
				   lcrzo_int16 level,
				   lcrzo_uint32 minnum,
				   lcrzo_uint32 maxnum,
				   lcrzo_uint32 *ptoolnumber)
{ lcrzo_uint32 toolnumber;

  lcrzo_epr(lcrzo_uint32_init_stdin((language==LCRZO_GLOBAL_LANG_FR)?
				    "Entrez votre numero d'outil":
				    "Enter your tool number", 
				    0, maxnum, minnum, &toolnumber));
  if (toolnumber==0) levels[level]='\0';
  if (ptoolnumber!=NULL) *ptoolnumber=toolnumber;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_subcat_choose(lcrzo_int16 language,
				     lcrzo_string levels,
				     lcrzo_int16 level,
				     char lastchar,
				     lcrzo_uint8 *pchoice)
{ lcrzo_uint8 choice, i;
  lcrzo_string str;

  lcrzo_epr(lcrzo_string_initm_text("0", &str));
  for ( i='a' ; i<lastchar ; i++ )
  { lcrzo_epr(lcrzo_string_appendm_char(i, 1, &str));
  }
  lcrzo_epr(lcrzo_stdin_char((language==LCRZO_GLOBAL_LANG_FR)?
			     "Entrez votre sous-categorie":
			     "Enter your sub-category", 
			     (lcrzo_constdata)str, 'a', &choice));
  lcrzo_string_free(str);
  if (choice=='0') levels[level-1]='\0';
  else { levels[level]=choice; levels[level+1]='\0'; }
  if (pchoice!=NULL) *pchoice=choice;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_helpchoice_tool_list(lcrzo_string levels, 
				 lcrzo_uint32 *ptoolnumber)
{ lcrzo_uint8 choice;
  lcrzo_uint32 toolnumber;
  lcrzo_int16 language, level;
  char c;

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  level=1;
  toolnumber=0;
  switch(levels[level])
  { case '\0':
      c='a';
      if ( language==LCRZO_GLOBAL_LANG_FR )
      { printf("\n########## MENU PRINCIPAL\n");
        printf(" 0 - quitter lcrzoex\n");
	printf(" %c - outils simples a utiliser (nouveaux utilisateurs, commencer ici)\n", c++);
	printf(" %c - obtient des informations\n", c++);
	printf(" %c - sniffe et sauve des paquets\n", c++);
	printf(" %c - affiche et rejoue des paquets sauves\n", c++);
	printf(" %c - spoof Ethernet et IP\n", c++);
	printf(" %c - clients udp\n", c++);
	printf(" %c - clients tcp\n", c++);
	printf(" %c - serveurs udp\n", c++);
	printf(" %c - serveurs tcp\n", c++);
	printf(" %c - routeurs\n", c++);
	printf(" %c - utilitaires pour administrateurs\n", c++);
	printf(" %c - serveur d'administration a distance\n", c++);
	printf(" %c - utilitaires malicieux\n", c++);
	printf(" %c - utilitaires divers\n", c++);
      }
      else
      { printf("\n########## MAIN MENU\n");
        printf(" 0 - quit lcrzoex\n");
	printf(" %c - easy tools (new users, start here)\n", c++);
	printf(" %c - obtains information\n", c++);
	printf(" %c - sniff and save packets\n", c++);
	printf(" %c - display and resend saved packets\n", c++);
	printf(" %c - Ethernet and IP spoof\n", c++);
	printf(" %c - udp clients\n", c++);
	printf(" %c - tcp clients\n", c++);
	printf(" %c - udp servers\n", c++);
	printf(" %c - tcp servers\n", c++);
	printf(" %c - routers\n", c++);
	printf(" %c - administrators utilities\n", c++);
	printf(" %c - remote administration server\n", c++);
	printf(" %c - malicious utilities\n", c++);
	printf(" %c - miscellaneous utilities\n", c++);
      }
      /* get the user's choice */
      lcrzo_epr(lcrzo_stdin_char((language==LCRZO_GLOBAL_LANG_FR)?
				 "Entrez votre categorie":
				 "Enter your category", 
				 (lcrzo_constdata)"0abcdefghijklmn",
				 'a', &choice));
      if (choice=='0') levels[level-1]='\0';
      else { levels[level]=choice; levels[level+1]='\0'; }
      break;

    case 'a' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000157));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000155));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000002));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000274));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000029));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000032));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000037));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000040));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000048));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000051));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000065));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000068));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000170));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000092));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000168));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000186));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000116));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000182));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000141));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000151));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000172));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
					       2, 274, &toolnumber));
      break;

    case 'b' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - informations sur cette machine\n", c++);
	    printf(" %c - informations sur une machine distante\n", c++);
	    printf(" %c - informations sur un reseau\n", c++);
	  }
	  else {
	    printf(" %c - information on this computer\n", c++);
	    printf(" %c - information on other computers\n", c++);
	    printf(" %c - information on this network\n", c++);
	  }
	  lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
						     c, &choice));
	  break;
        case 'a' :
	  lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000001));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000005));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000006));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000154));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000155));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000157));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000158));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000159));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000160));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000173));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000174));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000175));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000278));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
						   1, 278, &toolnumber));
	  break;
        case 'b' :
	  lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000002));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000156));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000003));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000004));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000176));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
						   2, 176, &toolnumber));
	  break;
        case 'c' :
	  lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000134));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000209));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000210));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000211));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000212));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
						   134, 212, &toolnumber));
	  break;
      }
      break;

    case 'c' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000007));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000274));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000275));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000008));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000009));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000295));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000296));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000297));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000010));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000011));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000012));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000013));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000014));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000015));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000016));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000017));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000018));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000019));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000020));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000021));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
					       2, 297, &toolnumber));
      break;

    case 'd' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000022));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000027));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000035));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000046));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000063));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000023));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000221));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000222));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000223));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000224));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000225));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000028));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000226));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000227));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000228));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000229));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000230));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000231));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000232));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000233));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000234));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000036));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000235));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000236));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000237));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000047));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000238));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000239));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000240));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000064));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000241));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000242));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000243));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000258));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000259));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
					       22, 259, &toolnumber));
      break;

    case 'e' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - spoof ethernet\n", c++);
	    printf(" %c - spoof ip\n", c++);
	    printf(" %c - spoof udp\n", c++);
	    printf(" %c - spoof tcp\n", c++);
	    printf(" %c - spoof icmp\n", c++);
	    printf(" %c - spoof arp/rarp\n", c++);
	  }
	  else {
	    printf(" %c - ethernet spoof\n", c++);
	    printf(" %c - ip spoof\n", c++);
	    printf(" %c - udp spoof\n", c++);
	    printf(" %c - tcp spoof\n", c++);
	    printf(" %c - icmp spoof\n", c++);
	    printf(" %c - arp/rarp spoof\n", c++);
	  }
	  lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
        case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000024));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000025));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000026));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000136));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   24, 136, &toolnumber));
          break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000029));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000298));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000030));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000031));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000164));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000032));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000299));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000033));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000034));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   29, 299, &toolnumber));
	  break;
        case 'c' :
	  level++;
	  switch(levels[level])
	  { case '\0':
              c='a';
	      lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	      if ( language==LCRZO_GLOBAL_LANG_FR ) { 
		printf(" %c - spoof udp generique\n", c++);
		printf(" %c - spoof udp syslog\n", c++);
		printf(" %c - spoof udp dns\n", c++);
	      }
	      else {
		printf(" %c - generic udp spoof\n", c++);
		printf(" %c - syslog udp spoof\n", c++);
		printf(" %c - dns udp spoof\n", c++);
	      }
	      lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, 
							 level,
							 c, &choice));
	      break;
	    case 'a' :
	      lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000037));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000300));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000038));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000039));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000040));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000301));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000041));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000042));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000135));
	      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, 
						       level, 
						       37, 135, &toolnumber));
	      break;
	    case 'b' :
	      lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000043));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000044));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000045));
	      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, 
						       level, 
						       43, 45, &toolnumber));
	      break;
	    case 'c' :
	      lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000327));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000328));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000329));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000330));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000331));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000332));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000333));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000334));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000335));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000336));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000337));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000338));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000339));
	      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000340));
	      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, 
						       level, 
						       327, 340, &toolnumber));
	      break;
	  }
	  break;
        case 'd' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000048));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000302));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000049));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000306));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000204));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000050));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000205));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000051));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000303));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000052));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000307));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000206));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000053));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000207));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000054));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000055));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000056));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000057));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000058));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000059));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000060));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000061));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000062));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   48, 307, &toolnumber));
	  break;
        case 'e' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000065));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000304));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000066));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000067));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000068));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000305));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000069));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000070));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000071));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000072));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000073));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000074));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000075));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000076));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000077));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000078));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000079));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   65, 305, &toolnumber));
	  break;
        case 'f' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000080));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000081));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000082));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000083));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000208));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   80, 208, &toolnumber));
	  break;
      }
      break;

    case 'f' :
      level++;
      switch(levels[level])
	{ case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - clients udp interactifs\n", c++);
	    printf(" %c - clients syslog\n", c++);
	    printf(" %c - clients dns\n", c++);
	    printf(" %c - clients dns (passe dans la categorie spoof)\n", c++);
	  }
	  else {
	    printf(" %c - udp interactive clients\n", c++);
	    printf(" %c - syslog clients\n", c++);
	    printf(" %c - dns clients\n", c++);
	    printf(" %c - dns clients (jump in spoof category)\n", c++);
	  }
          lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
	case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000084));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000308));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000085));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000168));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000309));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000178));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000086));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000087));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000169));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000179));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   84, 309, &toolnumber));
          break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000088));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000089));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000090));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000091));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   88, 91, &toolnumber));
          break;
	case 'c' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000341));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000342));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000343));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000344));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   341, 344, &toolnumber));
          break;
	case 'd' :
	  /* jump to this section */
	  strcpy(levels, "-ecc");
	  level=3;
          break;
        }
      break;

    case 'g' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - clients tcp interactifs\n", c++);
	    printf(" %c - clients tcp affichant la banniere d'accueil du serveur\n",
		   c++);
	    printf(" %c - clients HTTP (web)\n", c++);
	    printf(" %c - clients FTP\n", c++);
	    printf(" %c - clients SMTP (mail)\n", c++);
	    printf(" %c - clients NNTP (newsgroup)\n", c++);
	    printf(" %c - clients TELNET\n", c++);
	  }
	  else {
	    printf(" %c - tcp interactive clients\n", c++);
	    printf(" %c - clients displaying servers' banner\n", c++);
	    printf(" %c - HTTP (web) clients\n", c++);
	    printf(" %c - FTP clients\n", c++);
	    printf(" %c - SMTP (email) clients\n", c++);
	    printf(" %c - NNTP (newsgroup) clients\n", c++);
	    printf(" %c - TELNET clients\n", c++);
	  }
          lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
        case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000092));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000312));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000093));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000167));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000170));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000313));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000180));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000094));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000095));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000171));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000181));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   92, 313, &toolnumber));
	  break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000100));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000101));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000102));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000103));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000137));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   100, 137, &toolnumber));
	  break;
        case 'c' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000096));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000097));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000190));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000192));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000194));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000196));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000198));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000200));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000202));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000294));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000098));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000099));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000191));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000193));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000195));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000197));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000199));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000201));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000203));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   96, 294, &toolnumber));
	  break;
        case 'd' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000104));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000105));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000279));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000281));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000283));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000285));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000287));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000106));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000107));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000280));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000282));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000284));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000286));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000288));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000165));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000268));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000289));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000290));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000291));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000292));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   104, 292, &toolnumber));
	  break;
        case 'e' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000162));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000166));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000261));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000262));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000264));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   162, 264, &toolnumber));
	  break;
        case 'f' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000163));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000263));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000265));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000266));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000267));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000293));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   164, 293, &toolnumber));
	  break;
        case 'g' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000270));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000271));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000272));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000273));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   270, 273, &toolnumber));
	  break;
      }
      break;

    case 'h' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - serveurs udp interactifs\n", c++);
	    printf(" %c - serveurs udp monoclient\n", c++);
	    printf(" %c - serveurs udp multiclient\n", c++);
	    printf(" %c - serveurs DNS\n", c++);
	  }
	  else {
	    printf(" %c - udp interactive servers\n", c++);
	    printf(" %c - udp monoclient servers\n", c++);
	    printf(" %c - udp multiclient servers\n", c++);
	    printf(" %c - DNS servers\n", c++);
	  }
          lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
        case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000108));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000310));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000109));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000182));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000311));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000183));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000110));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000111));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000184));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000185));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   108, 311, &toolnumber));
          break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000112));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000113));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000114));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000115));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   112, 115, &toolnumber));
          break;
        case 'c' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000124));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000125));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000346));
          lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000126));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000347));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   124, 347, &toolnumber));
          break;
        case 'd' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000345));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000348));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000349));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000350));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   345, 350, &toolnumber));
          break;
      }
      break;

    case 'i' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - serveurs tcp interactifs\n", c++);
	    printf(" %c - serveurs tcp renvoyant les donnees recues\n", c++);
	  }
	  else {
	    printf(" %c - tcp interactive servers\n", c++);
	    printf(" %c - tcp servers echoing received data\n", c++);
	  }
          lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
        case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000116));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000314));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000117));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000186));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000315));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000187));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000118));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000119));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000188));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000189));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   116, 315, &toolnumber));
          break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000120));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000121));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000122));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000123));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000127));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000128));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   120, 128, &toolnumber));
          break;
      }
      break;

    case 'j' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000213));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000214));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                               213, 214, &toolnumber));
      break;

    case 'k' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000131));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000138));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000172));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000316));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000244));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000245));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000317));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000246));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000247));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000318));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000208));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000248));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000249));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000319));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000250));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000251));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000320));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000177));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                               131, 320, &toolnumber));
      break;

    case 'l' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000216));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000276));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000217));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000218));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000219));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000220));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000260));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000277));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                               216, 277, &toolnumber));
      break;

    case 'm' :
      level++;
      switch(levels[level])
      { case '\0':
          c='a';
          lcrzo_epr(lcrzoex_helpchoice_subcat(language));
	  if ( language==LCRZO_GLOBAL_LANG_FR ) { 
	    printf(" %c - attaque reseau\n", c++);
	    printf(" %c - email usurpe\n", c++);
	    printf(" %c - attaque brute force\n", c++);
	    printf(" %c - scanning\n", c++);
	  }
	  else {
	    printf(" %c - network attacks\n", c++);
	    printf(" %c - spoofed email\n", c++);
	    printf(" %c - brute force attack\n", c++);
	    printf(" %c - scanning\n", c++);
	  }
          lcrzo_epr(lcrzoex_helpchoice_subcat_choose(language, levels, level,
                                                     c, &choice));
          break;
        case 'a' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000129));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000130));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000132));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000269));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000133));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000321));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000137));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000215));
          lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   129, 321, &toolnumber));
          break;
        case 'b' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000166));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000262));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000264));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   166, 264, &toolnumber));
          break;
        case 'c' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000202));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000203));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000268));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000273));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   202, 273, &toolnumber));
          break;
        case 'd' :
          lcrzo_epr(lcrzoex_helpchoice_tool_subcat(language));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000252));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000253));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000322));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000254));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000255));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000323));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000256));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000257));
	  lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000324));
	  lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                                   252, 324, &toolnumber));
	  break;
      }
      break;

    case 'n' :
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(lcrzoex_helpchoice_tool_cat(language));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000139));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000140));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000141));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000142));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000143));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000144));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000161));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000145));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000146));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000147));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000148));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000149));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000150));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000151));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000152));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000153));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000325));
      lcrzo_epr(shared_lcrzoex_title_print(lcrzoex_000326));
      lcrzo_epr(lcrzoex_helpchoice_tool_choose(language, levels, level, 
                                               139, 326, &toolnumber));
      break;
  }

  if (ptoolnumber!=NULL) *ptoolnumber=toolnumber;
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/* this function classify and describes tools.
   this helps users to choose an tool */
int lcrzoex_helpchoice(lcrzo_conststring progname)
{ lcrzo_string pn;
  lcrzo_int8 levels[10];
  lcrzo_uint32 toolnumber;
  lcrzo_int16 language;

  /* get the current language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* check if user has admin privileges */
  lcrzo_epr(shared_misc_checkadmin(language));

  if ( language==LCRZO_GLOBAL_LANG_FR )
  { puts("Ces outils ont ete crees avec la bibliotheque lcrzo.");
  }
  else
  { puts("These tools were created with the network library lcrzo.");
  }

  /* loop through menus */
  levels[0]='-';
  levels[1]='\0';
  toolnumber=0;
  while ( levels[0] && (toolnumber==0) )
  { lcrzo_epr(lcrzoex_helpchoice_tool_list(levels, &toolnumber));
  }
  if (levels[0]=='\0') return(LCRZO_ERR_OK);

  /* display usage */
  if ( language==LCRZO_GLOBAL_LANG_FR )
    printf("\n---------- Voici comment utiliser cet outil :\n");
  else 
    printf("\n---------- Here is how to use this tool :\n");

  /* create a string containing "progname tool_number" */
  lcrzo_epr(lcrzo_string_initm_text(progname, &pn));
  lcrzo_epr(lcrzo_string_appendm_text(" ", &pn));
  lcrzo_epr(lcrzo_string_appendm_int(toolnumber, "%d", &pn));

  /* print infos about this tool */
  if ( toolnumber>=1 && toolnumber<=50 )
  { lcrzo_epr(lcrzoex__00001_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=51 && toolnumber<=100 )
  { lcrzo_epr(lcrzoex__00002_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=101 && toolnumber<=150 )
  { lcrzo_epr(lcrzoex__00003_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=151 && toolnumber<=200 )
  { lcrzo_epr(lcrzoex__00004_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=201 && toolnumber<=250 )
  { lcrzo_epr(lcrzoex__00005_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=251 && toolnumber<=300 )
  { lcrzo_epr(lcrzoex__00006_infos_print(pn, toolnumber));
  }
  else if ( toolnumber>=301 && toolnumber<=350 )
  { lcrzo_epr(lcrzoex__00007_infos_print(pn, toolnumber));
  }
  else
  { if ( language==LCRZO_GLOBAL_LANG_FR )
      lcrzo_int32_print("Choix '", toolnumber, "%d", "' inconnu\n");
    else
      lcrzo_int32_print("Unknown choice '", toolnumber, "%d", "'\n");
    lcrzo_string_free(pn);
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_string_free(pn);

  return(LCRZO_ERR_OK);
}
