/*
		                 LCRZOEX
                             Network toolbox
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzoex.

  Lcrzoex is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Lcrzoex is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details (http://www.gnu.org/).

------------------------------------------------------------------------
*/

#include <lcrzo.h>
#include "../lcrzoex.h"

/*-------------------------------------------------------------*/
int shared_http_initm_methurlhttpver(lcrzo_conststring method,
				     lcrzo_conststring url,
				     lcrzo_conststring version,
				     lcrzo_data *pdata,
				     lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* create "METHOD url HTTP/version" */
  lcrzo_epr(lcrzo_data_initm_text(method, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(url, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(" HTTP/", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(version, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_hostport(lcrzo_conststring hostname,
				 lcrzo_uint16 port,
				 lcrzo_int32 datacurrentsize,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* append "Host: hostname:port" */
  lcrzo_epr(lcrzo_data_appendm_text("Host: ", datacurrentsize, 
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(hostname, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(':', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(port, "%d", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_ipport(lcrzo_ipl ip,
			       lcrzo_uint16 port,
			       lcrzo_int32 datacurrentsize,
			       lcrzo_data *pdata,
			       lcrzo_int32 *pdatasize)
{ lcrzo_ips ips;

  lcrzo_epr(lcrzo_ips_init_ipl(ip, ips));
  lcrzo_epr(shared_http_appendm_hostport(ips, port, datacurrentsize, 
					 pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_authbasiclogpass(lcrzo_conststring login,
					 lcrzo_conststring passwd,
					 lcrzo_int32 datacurrentsize,
					 lcrzo_data *pdata,
					 lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string logpass;
 
  /* first, we generate the base64 of login:passwd */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(":", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(passwd, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_base64_initm_data(data, datasize, &logpass));
  lcrzo_data_free(data);

  /* append "Authorization: Basic base64_of_login:passwd" */
  lcrzo_epr(lcrzo_data_appendm_text("Authorization: Basic ", datacurrentsize,
                                    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(logpass, datasize, pdata, &datasize));
  lcrzo_string_free(logpass);
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_proxyauthbasiclogpass(lcrzo_conststring login,
					      lcrzo_conststring passwd,
					      lcrzo_int32 datacurrentsize,
					      lcrzo_data *pdata,
					      lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string logpass;
 
  /* first, we generate the base64 of login:passwd */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(":", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(passwd, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_base64_initm_data(data, datasize, &logpass));
  lcrzo_data_free(data);

  /* append "Proxy-Authorization: Basic base64_of_login:passwd" */
  lcrzo_epr(lcrzo_data_appendm_text("Proxy-Authorization: Basic ",
				    datacurrentsize,
                                    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(logpass, datasize, pdata, &datasize));
  lcrzo_string_free(logpass);
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_initm_httpvererr(lcrzo_conststring version,
				 lcrzo_int16 errorcode,
				 lcrzo_conststring errorstring,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* create "HTTP/version 000 message" */
  lcrzo_epr(lcrzo_data_initm_text("HTTP/", pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(version, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(errorcode, "%d", datasize,pdata,&datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(errorstring, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_connection(lcrzo_conststring type,
				   lcrzo_int32 datacurrentsize,
				   lcrzo_data *pdata,
				   lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("Connection: ", datacurrentsize,
				    pdata, &datasize));
  if (type==NULL)
  { lcrzo_epr(lcrzo_data_appendm_text("close", datasize, pdata, &datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text(type, datasize, pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_contenttype(lcrzo_conststring type,
				    lcrzo_int32 datacurrentsize,
				    lcrzo_data *pdata,
				    lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("Content-Type: ", datacurrentsize,
				    pdata, &datasize));
  if (type==NULL)
  { lcrzo_epr(lcrzo_data_appendm_text("text/html", datasize, pdata,&datasize));
  }
  else
  { lcrzo_epr(lcrzo_data_appendm_text(type, datasize, pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_wwwauth(lcrzo_conststring realm,
				lcrzo_int32 datacurrentsize,
				lcrzo_data *pdata,
				lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  lcrzo_epr(lcrzo_data_appendm_text("WWW-Authenticate: Basic realm=\"", 
				    datacurrentsize,
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(realm, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\"\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_percenttext_initm_data(lcrzo_constdata data,
				       lcrzo_int32 datasize,
				       lcrzo_string *ptextwithpercent)
{ lcrzo_string textwithpercent;
  char c;
  int i;

  lcrzo_epr(lcrzo_string_initm_text("", &textwithpercent));
  for ( i=0 ; i<datasize ; i++ ) 
  { c=data[i];
    if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z') || ( c>='A' && c<='Z' ) )
    { lcrzo_epr(lcrzo_string_appendm_char(c, 1, &textwithpercent));
    }
    else
    { lcrzo_epr(lcrzo_string_appendm_char('%', 1, &textwithpercent));
      lcrzo_epr(lcrzo_string_appendm_int(c, "%02X", &textwithpercent));
    }
  }
  
  if (ptextwithpercent==NULL) { lcrzo_string_free(textwithpercent); }
  else { *ptextwithpercent=textwithpercent; }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_data_initm_percenttext(lcrzo_conststring textwithpercent,
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize, textsize;
  char c, c1, c2;
  int i, number;

  lcrzo_epr(lcrzo_data_initm_text("", pdata, &datasize));
  textsize=strlen(textwithpercent);
  for ( i=0 ; i<textsize ; i++ ) 
  { c=textwithpercent[i];
    if ( c != '%' )
    { if ( c != '+' )
      { lcrzo_epr(lcrzo_data_appendm_char(c, 1, datasize, pdata, &datasize));
      }
      else
      { lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, pdata, &datasize));
      }
    }
    else
    { if ( i+2 >= textsize ) return(LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT);
      c1=textwithpercent[++i];
      c2=textwithpercent[++i];
      if ( c1>='0' && c1<='9' )
      { number=c1-'0';
      }
      else if ( c1>='a' && c1<='f') 
      { number=10+c1-'a';
      }
      else if ( c1>='A' && c1<='F' )
      { number=10+c1-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      number=number<<4;
      if ( c2>='0' && c2<='9' )
      { number+=c2-'0';
      }
      else if ( c2>='a' && c2<='f') 
      { number+=10+c2-'a';
      }
      else if ( c2>='A' && c2<='F' )
      { number+=10+c2-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      lcrzo_epr(lcrzo_data_appendm_char((char)number, 1, datasize, pdata,&datasize));
    }      
  }

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_get(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring proxylogin,
			    lcrzo_conststring proxypasswd,
			    lcrzo_conststring sitelogin,
			    lcrzo_conststring sitepasswd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("GET", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( proxylogin!=NULL && proxypasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, 
							proxypasswd, datasize,
							&data, &datasize));
  }
  if ( sitelogin!=NULL && sitepasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						   datasize, &data,&datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_post(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring proxylogin,
			     lcrzo_conststring proxypasswd,
			     lcrzo_conststring sitelogin,
			     lcrzo_conststring sitepasswd,
			     lcrzo_conststring parameters)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("POST", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( proxylogin!=NULL && proxypasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, 
							proxypasswd, datasize,
							&data, &datasize));
  }
  if ( sitelogin!=NULL && sitepasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						   datasize, &data,&datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text(
            "Content-Type: application/x-www-form-urlencoded\r\n",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(strlen(parameters), "%d", 
				   datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  /* now the parameters without an ending line */
  lcrzo_epr(lcrzo_data_appendm_text(parameters, datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_head(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring proxylogin,
			     lcrzo_conststring proxypasswd,
			     lcrzo_conststring sitelogin,
			     lcrzo_conststring sitepasswd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("HEAD", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( proxylogin!=NULL && proxypasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, 
							proxypasswd, datasize,
							&data, &datasize));
  }
  if ( sitelogin!=NULL && sitepasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						   datasize, &data,&datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_put(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring proxylogin,
			    lcrzo_conststring proxypasswd,
			    lcrzo_conststring sitelogin,
			    lcrzo_conststring sitepasswd,
			    lcrzo_data filedata,
			    lcrzo_int32 filedatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("PUT", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( proxylogin!=NULL && proxypasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, 
							proxypasswd, datasize,
							&data, &datasize));
  }
  if ( sitelogin!=NULL && sitepasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						   datasize, &data,&datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(filedatasize, "%d", 
				   datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  /* write the file to the socket */
  /*lcrzo_epr(lcrzo_data_print(filedata, filedatasize,LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, filedata, filedatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_delete(lcrzo_sock *psock,
			       lcrzo_conststring url,
			       lcrzo_conststring hostname,
			       lcrzo_uint16 port,
			       lcrzo_conststring proxylogin,
			       lcrzo_conststring proxypasswd,
			       lcrzo_conststring sitelogin,
			       lcrzo_conststring sitepasswd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("DELETE", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( proxylogin!=NULL && proxypasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_proxyauthbasiclogpass(proxylogin, 
							proxypasswd, datasize,
							&data, &datasize));
  }
  if ( sitelogin!=NULL && sitepasswd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(sitelogin, sitepasswd, 
						   datasize, &data,&datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_http_request_readheader(lcrzo_sock *psock,
				   lcrzo_time *pmaxwaitingtime,
				   lcrzo_string *pmethod,
				   lcrzo_string *purl,
				   lcrzo_string *pversion,
				   lcrzo_string *phost,
				   lcrzo_string *puseragent,
				   lcrzo_string *pproxylogin,
				   lcrzo_string *pproxypasswd,
				   lcrzo_string *psitelogin,
				   lcrzo_string *psitepasswd,
				   lcrzo_int32 *pcontentlength,
				   lcrzo_data *pheader,
				   lcrzo_int32 *pheadersize)
{ lcrzo_data data;
  lcrzo_int32 datasize, headersize;
  lcrzo_regexp re;
  lcrzo_bool hostfound, useragentfound, proxylogpassfound, sitelogpassfound;
  lcrzo_bool contentlengthfound;
  lcrzo_string authorization, authlogpass, integerstring;
  int ret;

  /* on the first line, there is "METHOD url HTTP/version" */
  lcrzo_epr(lcrzo_sock_readm_line(psock, pmaxwaitingtime, 
				  &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
			       "^([A-Za-z]+) (.*) HTTP/(...)$", re);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_data_free(data);
    return(LCRZOEX_ERR_HTTP_INVALIDREQUEST);
  }

  data[datasize]='\0';
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[1].startindex_positive,
				     re[1].endindex_positive, 
				     pmethod));
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[2].startindex_positive,
				     re[2].endindex_positive, 
				     purl));
  lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				     re[3].startindex_positive,
				     re[3].endindex_positive, 
				     pversion));
  lcrzo_epr(lcrzo_data_initm_data(data, datasize, pheader, &headersize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize, pheader, &headersize));
  lcrzo_data_free(data);

  /* to default values in case they are not set in the header */
  hostfound=LCRZO_FALSE;
  useragentfound=LCRZO_FALSE;
  proxylogpassfound=LCRZO_FALSE;
  sitelogpassfound=LCRZO_FALSE;
  contentlengthfound=LCRZO_FALSE;

  /* we read the header till the empty line */
  while (datasize)
  { /* read the line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, pmaxwaitingtime, 
				    &data, &datasize));
    /* add it to the returned header */     
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, headersize, 
				      pheader, &headersize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize,pheader,&headersize));
    /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
    if ( !hostfound )
    { /* search Host */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Host: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   phost));
        hostfound=LCRZO_TRUE;
      }
    }
    if ( !useragentfound )
    { /* search User-Agent */
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE);
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^User-Agent: (.+)$", re);
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   puseragent));
        useragentfound=LCRZO_TRUE;
      }
    }
    if ( !contentlengthfound )
    { /* search Content-Length */
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE);
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Length: ([0-9]+)$", re);
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &integerstring));
        if ( pcontentlength!=NULL ) *pcontentlength=atoi(integerstring);
	lcrzo_string_free(integerstring);
	contentlengthfound=LCRZO_TRUE;
      }
    }
    if ( !proxylogpassfound )
    { /* search Proxy-Authorization */
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE);
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Proxy-Authorization: Basic (.+)$", re);
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &authorization));
        lcrzo_epr(lcrzo_string_initm_base64(authorization, &authlogpass));
	lcrzo_string_free(authorization);
        ret=lcrzo_string_search_regexp(authlogpass, LCRZO_FALSE, +1, -1,
				       "^(.*):(.*)$", re);
        if ( ret==LCRZO_ERR_OK )
        { lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[1].startindex_positive,
					     re[1].endindex_positive, 
					     pproxylogin));
          lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[2].startindex_positive,
					     re[2].endindex_positive, 
					     pproxypasswd));
	  proxylogpassfound=LCRZO_TRUE;
	}
	lcrzo_string_free(authlogpass);
      }
    }
   if ( !sitelogpassfound )
    { /* search Authorization */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Authorization: Basic (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &authorization));
        lcrzo_epr(lcrzo_string_initm_base64(authorization, &authlogpass));
	lcrzo_string_free(authorization);
        ret=lcrzo_string_search_regexp(authlogpass, LCRZO_FALSE, +1, -1,
				       "^(.*):(.*)$", re);
        if ( ret==LCRZO_ERR_OK )
        { lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[1].startindex_positive,
					     re[1].endindex_positive, 
					     psitelogin));
          lcrzo_epr(lcrzo_string_initm_range(authlogpass, 
					     re[2].startindex_positive,
					     re[2].endindex_positive, 
					     psitepasswd));
	  sitelogpassfound=LCRZO_TRUE;
	}
	lcrzo_string_free(authlogpass);
      }
    }

    /* we do not need data */
    lcrzo_data_free(data);
  }

  /* set ouput values */
  if ( !hostfound && phost!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", phost));
  }
  if ( !useragentfound && puseragent!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", puseragent));
  }
  if ( !proxylogpassfound )
  { if (pproxylogin!=NULL) 
    { lcrzo_epr(lcrzo_string_initm_text("", pproxylogin));
    }
    if (pproxypasswd!=NULL) 
    { lcrzo_epr(lcrzo_string_initm_text("", pproxypasswd));
    }
  }
  if ( !sitelogpassfound )
  { if (psitelogin!=NULL) 
    { lcrzo_epr(lcrzo_string_initm_text("", psitelogin));
    }
    if (psitepasswd!=NULL) 
    { lcrzo_epr(lcrzo_string_initm_text("", psitepasswd));
    }
  }
  if ( !contentlengthfound && pcontentlength!=NULL )
  { *pcontentlength=0; 
  }
  if ( pheadersize!=NULL ) *pheadersize=headersize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_response_readheader(lcrzo_sock *psock,
				    lcrzo_int16 *perrornumber,
				    lcrzo_string *perrormsg,
				    lcrzo_string *pservertype,
				    lcrzo_string *pcontenttype,
				    lcrzo_int32 *pcontentlength,
				    lcrzo_data *pheader,
				    lcrzo_int32 *pheadersize)
{ lcrzo_data data;
  lcrzo_int32 datasize, headersize;
  lcrzo_string integerstring;
  lcrzo_regexp re;
  lcrzo_bool servertypefound, contenttypefound, contentlengthfound;
  int ret;

  /* on the first line, there is the error code */
  lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE, 
				  &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
                                 "HTTP/... ([0-9]+) (.+)", re);
  data[datasize]='\0';
  if ( ret==LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
				       re[1].startindex_positive,
				       re[1].endindex_positive, 
				       &integerstring));
    lcrzo_epr(lcrzo_int16_init_text(integerstring, perrornumber));
    lcrzo_string_free(integerstring);
    lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
				       re[2].startindex_positive,
				       re[2].endindex_positive, 
				       perrormsg));
  }
  lcrzo_epr(lcrzo_data_initm_data(data, datasize, pheader, &headersize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize, pheader, &headersize));
  lcrzo_data_free(data);

  /* to default values in case they are not set in the header */
  servertypefound=LCRZO_FALSE;
  contenttypefound=LCRZO_FALSE;
  contentlengthfound=LCRZO_FALSE;

  /* we read the header till the empty line */
  while (datasize)
  { /* read the line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_WAIT_INFINITE, 
				    &data, &datasize));
    /* add it to the returned header */     
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, headersize, 
				      pheader, &headersize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize,pheader,&headersize));
    /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
    if ( !servertypefound )
    { /* search Server */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Server: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pservertype));
        servertypefound=LCRZO_TRUE;
      }
    }
    if ( !contenttypefound )
    { /* search Content-Type */
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE);
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Type: (.+)$", re);
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data, 
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pcontenttype));
        contenttypefound=LCRZO_TRUE;
      }
    }
    if ( !contentlengthfound )
    { /* search Content-Length */
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_FALSE);
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Length: ([0-9]+)$", re);
      lcrzo_global_set_data_regexp_casesensitive(LCRZO_TRUE);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range((lcrzo_string)data,
					   re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &integerstring));
        if ( pcontentlength!=NULL ) *pcontentlength=atoi(integerstring);
	lcrzo_string_free(integerstring);
	contentlengthfound=LCRZO_TRUE;
      }
    }
    /* we do not need data */
    lcrzo_data_free(data);
  }

  /* set ouput values */
  if ( !servertypefound && pservertype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pservertype));
  }
  if ( !contenttypefound && pcontenttype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pcontenttype));
  }
  if ( !contentlengthfound && pcontentlength!=NULL )
  { *pcontentlength=-1; /* this value indicates that the header 
                           didn't contain the Content-Length line.
			   In this case (rfc2068), we have to read
			   till the server closes the connection :(
			*/
  }
  if ( pheadersize!=NULL ) *pheadersize=headersize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_response_readbody(lcrzo_sock *psock,
				  lcrzo_time *pmaxwaitingtime,
				  lcrzo_int32 contentlength,
				  lcrzo_data *pbody,
				  lcrzo_int32 *pbodysize)
{ lcrzo_data data;
  lcrzo_int32 datasize, bodysize;
  lcrzo_bool continuetoread;
  int ret;

  if ( contentlength != -1 )
  { lcrzo_er(lcrzo_sock_readm_fixed(psock, pmaxwaitingtime, 
				    contentlength, pbody));
    if ( pbodysize != NULL ) *pbodysize=contentlength;
  }
  else
  { /* -1 indicates that the header didn't contain the Content-Length line.
       In this case (rfc2068), we have to read till the server closes
       the connection :(
    */
    lcrzo_epr(lcrzo_data_initm_text("", pbody, &bodysize));
    continuetoread=LCRZO_TRUE;
    while (continuetoread)
    { ret=lcrzo_sock_readm(psock, pmaxwaitingtime, &data, &datasize);
      if (ret==LCRZO_ERR_OK)
      { lcrzo_epr(lcrzo_data_appendm_data(data, datasize, bodysize, 
					  pbody, &bodysize));
        lcrzo_epr(lcrzo_data_free(data));
      }
      else if (ret==LCRZO_ERR_OKDATAEND)
      { if ( pbodysize != NULL ) *pbodysize=bodysize;
        continuetoread=LCRZO_FALSE;
      }
      else
	return(ret);
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_set_proxysite_userpass(int argc, 
				       char *argv[],
				       int startat,
				       lcrzo_string *pproxyuser, 
				       lcrzo_string *pproxypasswd, 
				       lcrzo_string *psiteuser,
				       lcrzo_string *psitepasswd)
{ 
  if (pproxyuser!=NULL) *pproxyuser=NULL;
  if (psiteuser!=NULL) *psiteuser=NULL;
  if (pproxypasswd!=NULL) *pproxypasswd=NULL;
  if (psitepasswd!=NULL) *psitepasswd=NULL;

  /* get proxy login */
  if ( argc>=startat+1 ) 
  { if (strlen(argv[startat]))
    { lcrzo_epr(shared_para_string_init(argv[startat], "Proxy login ", "",
					pproxyuser));
    }
  }

  /* get site login */
  if ( argc>=startat+2 ) 
  { if (strlen(argv[startat+1]))
    { lcrzo_epr(shared_para_string_init(argv[startat+1], "Site login ", "",
					psiteuser));
    }
  }

  /* get proxy password */
  if ( argc>=startat+3 )
  { if (strlen(argv[startat+2]))
    { lcrzo_epr(shared_para_string_init(argv[startat+2], "Proxy password ", 
					"", pproxypasswd));
    }
  }

  /* get site password */
  if ( argc>=startat+4 )
  { if (strlen(argv[startat+3]))
    { lcrzo_epr(shared_para_string_init(argv[startat+3], "site password ", 
					"", pproxypasswd));
    }
  }

  /* error cases */
  if ( pproxyuser!=NULL && *pproxyuser==NULL &&
       pproxypasswd!=NULL && *pproxypasswd!=NULL )
  { puts("Error : proxypasswd set, but proxyuser not set");
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  if ( psiteuser!=NULL && *psiteuser==NULL &&
       psitepasswd!=NULL && *psitepasswd!=NULL )
  { puts("Error : sitepasswd set, but siteuser not set");
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }

  /* get missing proxy passwd */
  if ( pproxyuser!=NULL && *pproxyuser!=NULL &&
       pproxypasswd!=NULL && *pproxypasswd==NULL )
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Proxy passwd", "",
					     pproxypasswd));
  }

  /* get missing site passwd */
  if ( psiteuser!=NULL && *psiteuser!=NULL &&
       psitepasswd!=NULL && *psitepasswd==NULL )
  { lcrzo_epr(lcrzo_string_initm_text_passwd("Site passwd", "",
					     psitepasswd));
  }

  /*
  if (pproxyuser!=NULL && *pproxyuser!=NULL) printf("PU : %s\n", *pproxyuser);
  if (psiteuser!=NULL && *psiteuser!=NULL) printf("SU : %s\n", *psiteuser);
  if (pproxypasswd!=NULL && *pproxypasswd!=NULL)
    printf("PP : %s\n", *pproxypasswd);
  if (psitepasswd!=NULL && *psitepasswd!=NULL)
    printf("SP : %s\n", *psitepasswd);
  */

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_get_variablevalue(lcrzo_conststring url,
				  lcrzo_conststring variablename,
				  lcrzo_data *pvariablevalue,
				  lcrzo_int32 *pvariablevaluesize)
{ lcrzo_string searchstring, variablevalue;
  lcrzo_regexp re;
  int ret, i;

  if (url==NULL || variablename==NULL) return(LCRZO_ERR_PANULLPTR);

  for ( i=0 ; i<4 ; i++ )
  {
    /* initialize the searched variable */
    if ( i==0 || i==2 )
    { lcrzo_epr(lcrzo_string_initm_text("^", &searchstring));
    }
    else
    { lcrzo_epr(lcrzo_string_initm_text("[&?]", &searchstring));
    }
    lcrzo_epr(lcrzo_string_appendm_text(variablename, &searchstring));
    lcrzo_epr(lcrzo_string_appendm_text("=(.*)", &searchstring));
    if ( i==0 || i==1 )
    { lcrzo_epr(lcrzo_string_appendm_text("$", &searchstring));
    }
    else
    { lcrzo_epr(lcrzo_string_appendm_text("[&?]", &searchstring));
    }

    /* search the variable */
    ret=lcrzo_string_search_regexp(url, LCRZO_FALSE, +1, -1, searchstring, re);
    lcrzo_string_free(searchstring);
    if ( ret==LCRZO_ERR_OK )
    { lcrzo_epr(lcrzo_string_initm_range(url, re[1].startindex_positive,
					 re[1].endindex_positive, 
					 &variablevalue));
      ret=shared_http_data_initm_percenttext(variablevalue, pvariablevalue,
					     pvariablevaluesize);
      lcrzo_string_free(variablevalue);
      return(ret);
    }
  }

  /* set an empty value */
  lcrzo_epr(lcrzo_data_initm_text("", pvariablevalue, pvariablevaluesize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/* Note : we do not check for allowed characters */
int shared_http_url_get_parts(lcrzo_conststring url, 
			      lcrzo_string *pproto, 
			      lcrzo_string *plogin,
			      lcrzo_string *ppasswd, 
			      lcrzo_string *pserver, 
			      lcrzo_uint16 *pport,
			      lcrzo_string *pfile)
{ lcrzo_int32 posbegin, posend, pos;
  lcrzo_string mainblock, loginpass, serverport, portstr, proto;
  int ret;

  /* special case */
  if ( url==NULL || strlen(url)==0 )
  { lcrzo_er(lcrzo_string_initm_text("", pproto));
    lcrzo_er(lcrzo_string_initm_text("", plogin));
    lcrzo_er(lcrzo_string_initm_text("", ppasswd));
    lcrzo_er(lcrzo_string_initm_text("", pserver));
    if (pport!=NULL) *pport=0;
    lcrzo_er(lcrzo_string_initm_text("", pfile));
    return (LCRZO_ERR_OK);
  }

  /* search "://" */
  ret=lcrzo_string_search_text(url, LCRZO_FALSE, +1, -1, "://", 
			       NULL, &posbegin, NULL, &posend);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(url, +1, posbegin, &proto));
    lcrzo_er(lcrzo_string_initm_range(url, posend, -1, &mainblock));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text("http", &proto));
    lcrzo_er(lcrzo_string_initm_text(url, &mainblock));
  }
  lcrzo_er(lcrzo_string_initm_text(proto, pproto));
  
  /* search "/" */
  ret=lcrzo_string_search_char(mainblock, LCRZO_FALSE, +1,-1, '/', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(mainblock, pos, -1, pfile));
    mainblock[pos-1]='\0';
  }
  else
  { lcrzo_er(lcrzo_string_initm_text("/", pfile));
  }

  /* mainblock now contains login:pass@server:port */
  /* search "@" */
  ret=lcrzo_string_search_char(mainblock, LCRZO_FALSE, +1,-1, '@', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(mainblock, +1, pos, &loginpass));
    lcrzo_er(lcrzo_string_initm_range(mainblock, pos+1, -1, &serverport));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text("", &loginpass));
    lcrzo_er(lcrzo_string_initm_text(mainblock, &serverport));
  }
  lcrzo_er(lcrzo_string_free(mainblock));

  /* search ":" */
  ret=lcrzo_string_search_char(loginpass, LCRZO_FALSE, +1,-1, ':', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(loginpass, +1, pos, plogin));
    lcrzo_er(lcrzo_string_initm_range(loginpass, pos+1, -1, ppasswd));
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(loginpass, plogin));
    lcrzo_er(lcrzo_string_initm_text("", ppasswd));
  }
  lcrzo_er(lcrzo_string_free(loginpass));

  /* search ":" */
  ret=lcrzo_string_search_char(serverport, LCRZO_FALSE, +1,-1, ':', NULL,&pos);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_er(lcrzo_string_initm_range(serverport, +1, pos, pserver));
    lcrzo_er(lcrzo_string_initm_range(serverport, pos+1, -1, &portstr));
    ret=lcrzo_uint16_init_text(portstr, pport);
    if (ret!=LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_string_free(serverport));
      lcrzo_er(lcrzo_string_free(portstr));
      lcrzo_er(lcrzo_string_free(proto));
      return(ret);
    }
  }
  else
  { lcrzo_er(lcrzo_string_initm_text(serverport, pserver));
    /* try to find the port to use */
    if ( !shared_misc_strcasecmp(proto, "http") )
    { if (pport!=NULL) *pport=80;
    }
    else if ( !shared_misc_strcasecmp(proto, "ftp") )
    { if (pport!=NULL) *pport=21;
    }
    else
    { if (pport!=NULL) *pport=0;
    }
  }
  lcrzo_er(lcrzo_string_free(serverport));
  lcrzo_er(lcrzo_string_free(proto));

  /*printf("%s -> %s :// %s : %s @ %s : %d / %s\n", url, *pproto, *plogin,
   *ppasswd, *pserver, *pport, *pfile);*/

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_priv_http_url_download_file(lcrzo_conststring server, 
				       lcrzo_uint16 port,
				       lcrzo_conststring url,
				       lcrzo_conststring proxylogin,
				       lcrzo_conststring proxypasswd,
				       lcrzo_conststring sitelogin,
				       lcrzo_conststring sitepasswd, 
				       lcrzo_conststring localfile);
int shared_priv_http_url_download_file(lcrzo_conststring server, 
				       lcrzo_uint16 port,
				       lcrzo_conststring url,
				       lcrzo_conststring proxylogin,
				       lcrzo_conststring proxypasswd,
				       lcrzo_conststring sitelogin,
				       lcrzo_conststring sitepasswd, 
				       lcrzo_conststring localfile)
{ lcrzo_ipl ipl;
  lcrzo_sock sock;
  lcrzo_int16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header, body;
  lcrzo_int32 headersize, contentlength, bodysize;

  /* open the connection */
  lcrzo_epr(lcrzo_ipl_init_hs(server, &ipl));
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_epr(shared_http_request_get(&sock, url, server, port,
				    proxylogin, proxypasswd, 
				    sitelogin, sitepasswd));

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength,
					    &header, &headersize));
  /*  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  lcrzo_int32_print("Content-length: ", contentlength, "%d", "\n");
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  */
  lcrzo_string_free(errormsg);
  lcrzo_string_free(servertype);
  lcrzo_string_free(contenttype);
  lcrzo_data_free(header);

  /* get the body */
  if (contentlength)
  { lcrzo_epr(shared_http_response_readbody(&sock, LCRZO_WAIT_INFINITE,
					    contentlength, 
					    &body, &bodysize));
    lcrzo_epr(lcrzo_filename_write(localfile, LCRZO_FALSE, LCRZO_FALSE,
				   body, bodysize));
    lcrzo_data_free(body);
  }

  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_url_download_file(lcrzo_conststring siteurl, 
				  lcrzo_conststring proxyurl, 
				  lcrzo_conststring localfile)
{ lcrzo_string url;
  lcrzo_uint16 siteport, proxyport;
  lcrzo_string siteproto, sitelogin, sitepasswd, siteserver, sitefile;
  lcrzo_string proxyproto, proxylogin, proxypasswd, proxyserver, proxyfile;
 
  /* extract parts of the url */
  lcrzo_epr(shared_http_url_get_parts(siteurl, &siteproto, &sitelogin,
				      &sitepasswd, &siteserver, &siteport,
				      &sitefile)); 
  lcrzo_epr(shared_http_url_get_parts(proxyurl, &proxyproto, &proxylogin,
				      &proxypasswd, &proxyserver, &proxyport,
				      &proxyfile)); 

  if ( !shared_misc_strcasecmp(siteproto, "http") )
  { if (strlen(proxyserver))
    { /* use proxy */ 
      lcrzo_er(lcrzo_string_initm_text("http://", &url));
      lcrzo_er(lcrzo_string_appendm_text(siteserver, &url));
      lcrzo_er(lcrzo_string_appendm_text(":", &url));
      lcrzo_er(lcrzo_string_appendm_int(siteport, "%d", &url));
      lcrzo_er(lcrzo_string_appendm_text(sitefile, &url));
      lcrzo_epr(shared_priv_http_url_download_file(proxyserver, proxyport,
						   url,
						   proxylogin, proxypasswd,
						   sitelogin, sitepasswd, 
						   localfile));
      lcrzo_er(lcrzo_string_free(url));
    }
    else
    { /* connect to the server */
      lcrzo_epr(shared_priv_http_url_download_file(siteserver, siteport,
						   sitefile,
						   NULL, NULL,
						   sitelogin, sitepasswd, 
						   localfile));
    }
  }

  /* free variables */
  lcrzo_string_free(siteproto);
  lcrzo_string_free(sitelogin);
  lcrzo_string_free(sitepasswd);
  lcrzo_string_free(siteserver);
  lcrzo_string_free(sitefile);
  lcrzo_string_free(proxyproto);
  lcrzo_string_free(proxylogin);
  lcrzo_string_free(proxypasswd);
  lcrzo_string_free(proxyserver);
  lcrzo_string_free(proxyfile);

  return(LCRZO_ERR_OK);
}
